/* 
 * Copyright (C) 2000-2001 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: mrlbrowser.c,v 1.9 2001/08/31 19:04:31 jkeil Exp $
 *
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef NEED_MRLBROWSER

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>                                                      
#include <assert.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

#include "_xitk.h"
#include "mrlbrowser.h"
#include "browser.h"
#include "label.h"
#include "button.h"
#include "labelbutton.h"
#include "dnd.h"
#include "widget.h"
#include "widget_types.h"

#include "xine.h"

/*
 * Redisplay the displayed current origin (useful for file plugin).
 */
static void update_current_origin(mrlbrowser_private_data_t *private_data) {

  if(private_data->mc->mrls[0] && private_data->mc->mrls[0]->origin)
    sprintf(private_data->current_origin, "%s", 
	    private_data->mc->mrls[0]->origin);
  else
    sprintf(private_data->current_origin, "%s", "");
  
  label_change_label (private_data->widget_list, 
		      private_data->widget_origin, 
		      private_data->current_origin);
  
}

/*
 * Return the *marker* of the giver file *fully named*
 */
static int get_mrl_marker(mrl_t *mrl) {
  
  if(mrl->type & mrl_file_symlink)
    return '@';
  else if(mrl->type & mrl_file_directory)
    return '/';
  else if(mrl->type & mrl_file_fifo)
    return '|';
  else if(mrl->type & mrl_file_sock)
    return '=';
  else if(mrl->type & mrl_file_normal) {
    if(mrl->type & mrl_file_exec)
      return '*';
  }

  return '\0';
}

/*
 * Create *eye candy* entries in browser.
 */
static void mrlbrowser_create_enlighted_entries(mrlbrowser_private_data_t *private_data) {
  int i = 0;
  
  while(private_data->mc->mrls[i]) {
    char *p = private_data->mc->mrls[i]->mrl;

    if(private_data->mc->mrls[i]->origin) {

      p += strlen(private_data->mc->mrls[i]->origin);
      
      if(*p == '/') 
	p++;
    }
    
    if(private_data->mc->mrls[i]->type & mrl_file_symlink) {

      if(private_data->mc->mrls_disp[i])
	private_data->mc->mrls_disp[i] = (char *) 
	  realloc(private_data->mc->mrls_disp[i], strlen(p) + 4 + 
		  strlen(private_data->mc->mrls[i]->link) + 2);
      else
	private_data->mc->mrls_disp[i] = (char *) 
	  gui_xmalloc(strlen(p) + 4 + 
		      strlen(private_data->mc->mrls[i]->link) + 2);
      
      sprintf(private_data->mc->mrls_disp[i], "%s%c -> %s", 
	      p, get_mrl_marker(private_data->mc->mrls[i]), 
	      private_data->mc->mrls[i]->link);

    }
    else {
      if(private_data->mc->mrls_disp[i])
	private_data->mc->mrls_disp[i] = (char *) 
	  realloc(private_data->mc->mrls_disp[i], strlen(p) + 2);
      else
      private_data->mc->mrls_disp[i] = (char *) gui_xmalloc(strlen(p) + 2);
      
      sprintf(private_data->mc->mrls_disp[i], "%s%c", 
	      p, get_mrl_marker(private_data->mc->mrls[i]));
    }

    i++;
  }
}

/*
 * Duplicate mrls from xine-engine's returned. We shouldn't play
 * directly with these ones.
 */
static void mrlbrowser_duplicate_mrls(mrlbrowser_private_data_t *private_data,
				      mrl_t **mtmp, int num_mrls) {
  int i;
  int old_mrls_num = private_data->mrls_num;

  for (i=0; i<num_mrls; i++) {

    if(private_data->mc->mrls[i] == NULL)
      private_data->mc->mrls[i] = (mrl_t *) gui_xmalloc(sizeof(mrl_t));

    MRL_DUPLICATE(mtmp[i], private_data->mc->mrls[i]);
  }

  private_data->mrls_num = i;

  while(old_mrls_num > private_data->mrls_num) {
    MRL_ZERO(private_data->mc->mrls[old_mrls_num-1]);
    free(private_data->mc->mrls[old_mrls_num-1]);
    private_data->mc->mrls[old_mrls_num-1] = NULL;
    old_mrls_num--;
  }

  private_data->mc->mrls[private_data->mrls_num] = NULL;
}

/*
 * Grab mrls from xine-engine.
 */
static void mrlbrowser_grab_mrls(widget_t *w, void *data) {
  mrlbrowser_private_data_t *private_data = (mrlbrowser_private_data_t *)data;
  char *lbl = (char *) labelbutton_get_label(w);
  char *old_old_src;

  if(lbl) {
    
    if(private_data->last_mrl_source)
      private_data->last_mrl_source = (char *)
	realloc(private_data->last_mrl_source, strlen(lbl) + 1);
    else
      private_data->last_mrl_source = (char *) gui_xmalloc(strlen(lbl) + 1);
    
    old_old_src = strdup(private_data->last_mrl_source);
    sprintf(private_data->last_mrl_source, "%s", lbl);

    {
      int num_mrls;
      mrl_t **mtmp = xine_get_browse_mrls(private_data->xine, 
					  private_data->last_mrl_source, NULL,
					  &num_mrls);
      
      if(!mtmp) {
	private_data->last_mrl_source = (char *)
	  realloc(private_data->last_mrl_source, strlen(old_old_src) + 1);
	sprintf(private_data->last_mrl_source, "%s", old_old_src);
	return;
      }

      mrlbrowser_duplicate_mrls(private_data, mtmp, num_mrls);
    }
    
    update_current_origin(private_data);
    mrlbrowser_create_enlighted_entries(private_data);
    browser_update_list(private_data->mrlb_list, 
			private_data->mc->mrls_disp, 
			private_data->mrls_num, 0);
  }
}

/*
 * Dump informations, on terminal, about selected mrl.
 */
static void mrlbrowser_dumpmrl(widget_t *w, void *data) {
  mrlbrowser_private_data_t *private_data = (mrlbrowser_private_data_t *)data;
  int j = -1;
  
  if((j = browser_get_current_selected(private_data->mrlb_list)) >= 0) {
    mrl_t *ms = private_data->mc->mrls[j];
    
    printf("mrl '%s'\n\t+", ms->mrl);

    if(ms->type & mrl_file_symlink)
      printf(" symlink to '%s' \n\t+ ", ms->link);

    printf("[");

    if(ms->type & mrl_unknown)
      printf(" unknown ");
    
      if(ms->type & mrl_dvd)
	printf(" dvd ");
      
      if(ms->type & mrl_vcd)
	    printf(" vcd ");
      
      if(ms->type & mrl_net)
	printf(" net ");
      
      if(ms->type & mrl_rtp)
	printf(" rtp ");
      
      if(ms->type & mrl_stdin)
	printf(" stdin ");
      
      if(ms->type & mrl_file)
	printf(" file ");
      
      if(ms->type & mrl_file_fifo)
	printf(" fifo ");
      
      if(ms->type & mrl_file_chardev)
	printf(" chardev ");
      
      if(ms->type & mrl_file_directory)
	printf(" directory ");
      
      if(ms->type & mrl_file_blockdev)
	printf(" blockdev ");
      
      if(ms->type & mrl_file_normal)
	printf(" normal ");
      
      if(ms->type & mrl_file_sock)
	printf(" sock ");
      
      if(ms->type & mrl_file_exec)
	printf(" exec ");
      
      if(ms->type & mrl_file_backup)
	printf(" backup ");
      
      if(ms->type & mrl_file_hidden)
	printf(" hidden ");
      
      printf("] (%Ld byte%c)\n", ms->size, (ms->size > 0) ?'s':'\0');
  }
}
/*
 *                                 END OF PRIVATES
 *****************************************************************************/

/*
 * Fill window information struct of given widget.
 */
int mrlbrowser_get_window_info(widget_t *w, window_info_t *inf) {
  mrlbrowser_private_data_t *private_data = 
    (mrlbrowser_private_data_t *)w->private_data;
  
  return((widget_get_window_info(private_data->widget_key, inf))); 
}

/*
 * Boolean about running state.
 */
int mrlbrowser_is_running(widget_t *w) {
  mrlbrowser_private_data_t *private_data;
 
  if(w) {
    private_data = w->private_data;
    return (private_data->running);
  }

  return 0;
}

/*
 * Boolean about visible state.
 */
int mrlbrowser_is_visible(widget_t *w) {
  mrlbrowser_private_data_t *private_data;

  if(w) {
    private_data = w->private_data;
    return (private_data->visible);
  }

  return 0;
}

/*
 * Hide mrlbrowser.
 */
void mrlbrowser_hide(widget_t *w) {
  mrlbrowser_private_data_t *private_data;

  if(w) {
    private_data = w->private_data;

    if(private_data->visible) {
      XLOCK(private_data->display);
      XUnmapWindow(private_data->display, private_data->window);
      widget_hide_widgets(private_data->widget_list);
      XUNLOCK(private_data->display);
      private_data->visible = 0;
    }
  }
}

/*
 * Show mrlbrowser.
 */
void mrlbrowser_show(widget_t *w) {
  mrlbrowser_private_data_t *private_data;

  if(w) {
    private_data = w->private_data;

    XLOCK(private_data->display);
    widget_show_widgets(private_data->widget_list);
    XMapRaised(private_data->display, private_data->window); 
    XUNLOCK(private_data->display);
    private_data->visible = 1;
  }
}

/*
 * Set mrlbrowser transient for hints for given window.
 */
void mrlbrowser_set_transient(widget_t *w, Window window) {
  mrlbrowser_private_data_t *private_data;

  if(w && (window != None)) {
    private_data = w->private_data;

    if(private_data->visible) {
      XLOCK(private_data->display);
      XSetTransientForHint (private_data->display,
			    private_data->window, 
			    window);
      XUNLOCK(private_data->display);
    }
  }
}

/*
 * Destroy the mrlbrowser.
 */
void mrlbrowser_destroy(widget_t *w) {
  mrlbrowser_private_data_t *private_data;

  if(w) {
    private_data = w->private_data;

    private_data->running = 0;
    private_data->visible = 0;
    
    XLOCK(private_data->display);
    XUnmapWindow(private_data->display, private_data->window);
    XUNLOCK(private_data->display);
   
    widget_stop_widgets(private_data->widget_list);
    gui_list_free(private_data->widget_list->l);
    free(private_data->widget_list);

    {
      int i;
      for(i = private_data->mrls_num; i > 0; i--) {
	MRL_ZERO(private_data->mc->mrls[i]);
	free(private_data->mc->mrls[i]);
	free(private_data->mc->mrls_disp[i]);
      }
    }
    
    free(private_data->mc);

    XLOCK(private_data->display);
    XDestroyWindow(private_data->display, private_data->window);
    XUNLOCK(private_data->display);

    widget_unregister_event_handler(&private_data->widget_key);
    free(private_data->fbWidget);
    free(private_data);
  }
}

/*
 * Leaving mrlbrowser.
 */
void mrlbrowser_exit(widget_t *w, void *data) {
  mrlbrowser_private_data_t *private_data = ((widget_t *)data)->private_data;
  
  if(private_data->kill_callback)
    private_data->kill_callback(private_data->fbWidget, NULL);

  private_data->running = 0;
  private_data->visible = 0;
  
  XLOCK(private_data->display);
  XUnmapWindow(private_data->display, private_data->window);
  XUNLOCK(private_data->display);

  widget_stop_widgets(private_data->widget_list);
  gui_list_free(private_data->widget_list->l);
  free(private_data->widget_list);

  {
    int i;
    for(i = private_data->mrls_num; i > 0; i--) {
      MRL_ZERO(private_data->mc->mrls[i]);
      free(private_data->mc->mrls[i]);
      free(private_data->mc->mrls_disp[i]);
    }
  }

  free(private_data->mc);
  
  XLOCK(private_data->display);
  XDestroyWindow(private_data->display, private_data->window);
  XUNLOCK(private_data->display);

  widget_unregister_event_handler(&private_data->widget_key);
  free(private_data->fbWidget);
  free(private_data);
}

/*
 * 
 */
static void mrlbrowser_select_mrl(mrlbrowser_private_data_t *private_data,
				  int j) {
  mrl_t *ms = private_data->mc->mrls[j];
  char   buf[PATH_MAX + NAME_MAX + 1];
  
  memset(&buf, '\0', sizeof(buf));
  sprintf(buf, "%s", ms->mrl);
  
  if((ms->type & mrl_file) && (ms->type & mrl_file_directory)) {
    char *filename = ms->mrl;
    
    if(ms->origin) {
      filename += strlen(ms->origin);
      
      if(*filename == '/') 
	filename++;
    }
    
    /* Check if we want to re-read current dir or go in parent dir */
    if((filename[strlen(filename) - 1] == '.') &&
       (filename[strlen(filename) - 2] != '.')) {
      
      memset(&buf, '\0', sizeof(buf));
      sprintf(buf, "%s", ms->origin);
    }
    else if((filename[strlen(filename) - 1] == '.') &&
	    (filename[strlen(filename) - 2] == '.')) {
      char *p;
      
      memset(&buf, '\0', sizeof(buf));
      sprintf(buf, "%s", ms->origin);
      if(strlen(buf) > 1) { /* not '/' directory */
	  
	p = &buf[strlen(buf)-1];
	while(*p && *p != '/') {
	  *p = '\0';
	    p--;
	}
	
	/* Remove last '/' if current_dir isn't root */
	if((strlen(buf) > 1) && *p == '/') 
	  *p = '\0';
	  
      }
    }
    
    {
      int num_mrls;
      mrl_t **mtmp = xine_get_browse_mrls(private_data->xine, 
					  private_data->last_mrl_source, 
					  buf, &num_mrls);
      
      mrlbrowser_duplicate_mrls(private_data, mtmp, num_mrls);
    }
    
    update_current_origin(private_data);
    mrlbrowser_create_enlighted_entries(private_data);
    browser_update_list(private_data->mrlb_list, 
			private_data->mc->mrls_disp, 
			private_data->mrls_num, 0);
    
  }
  else {
    
    browser_release_all_buttons(private_data->mrlb_list);
    if(private_data->add_callback)
      private_data->add_callback(NULL, (void *) j, 
				 private_data->mc->mrls[j]);

  }
}

/*
 * Handle selection in mrlbrowser.
 */
static void mrlbrowser_select(widget_t *w, void *data) {
  mrlbrowser_private_data_t *private_data = (mrlbrowser_private_data_t *)data;
  int j = -1;

  if((j = browser_get_current_selected(private_data->mrlb_list)) >= 0) {
    mrlbrowser_select_mrl(private_data, j);
  }
}

/*
 * Handle double click in labelbutton list.
 */
static void handle_dbl_click(widget_t *w, void *data, int selected) {
  mrlbrowser_private_data_t *private_data = (mrlbrowser_private_data_t *)data;

  mrlbrowser_select_mrl(private_data, selected);
}

/*
 * Handle here mrlbrowser events.
 */
static void mrlbrowser_handle_event(XEvent *event, void *data) {
  mrlbrowser_private_data_t *private_data = (mrlbrowser_private_data_t *)data;
  XKeyEvent      mykeyevent;
  KeySym         mykey;
  char           kbuf[256];
  int            len;
  
  switch(event->type) {

  case MappingNotify:
    XLOCK(private_data->display);
    XRefreshKeyboardMapping((XMappingEvent *) event);
    XUNLOCK(private_data->display);
    break;
    
  case KeyPress:
    mykeyevent = event->xkey;
    
    XLOCK (private_data->display);
    len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
    XUNLOCK (private_data->display);
    
    switch (mykey) {
      
    case XK_s:
    case XK_S:
      mrlbrowser_select(NULL, (void *)private_data);
      break;
      
      /* This is for debugging purpose */
    case XK_d:
    case XK_D:
      mrlbrowser_dumpmrl(NULL, (void *)private_data);
      break;
      
    case XK_Down:
    case XK_Next:
      browser_step_up((widget_t *)private_data->mrlb_list, NULL);
      break;
      
    case XK_Up:
    case XK_Prior:
      browser_step_down((widget_t *)private_data->mrlb_list, NULL);
      break;
    }
    
    break;
    
  }
}

/*
 * Create mrlbrowser window.
 */
widget_t *mrlbrowser_create(xitk_mrlbrowser_t *mb) {
  GC                         gc;
  XSizeHints                 hint;
  XSetWindowAttributes       attr;
  char                      *title = mb->window_title;
  Atom                       prop;
  MWMHints                   mwmhints;
  XWMHints                  *wm_hint;
  XClassHint                *xclasshint;
  XColor                     black, dummy;
  int                        screen;
  widget_t                  *mywidget;
  mrlbrowser_private_data_t *private_data;
  xitk_labelbutton_t         lb;
  xitk_label_t               lbl;
  
  if(mb->ip_availables == NULL) {
    fprintf(stderr, "Something's going wrong, there is no input plugin "
	    "available having INPUT_CAP_GET_DIR capability !!\nExiting.\n");
    exit(1);
  }

  if(mb->xine == NULL) {
    fprintf(stderr, "Xine engine should be initialized first !!\nExiting.\n");
    exit(1);
  }

  mywidget               = (widget_t *) gui_xmalloc(sizeof(widget_t));
  
  private_data           = (mrlbrowser_private_data_t *) 
    gui_xmalloc(sizeof(mrlbrowser_private_data_t));

  mywidget->private_data = private_data;

  private_data->fbWidget = mywidget;
  private_data->display  = mb->display;
  private_data->xine     = mb->xine;
  private_data->running  = 1;

  XLOCK(mb->display);

  if(!(private_data->bg_image = Imlib_load_image(mb->imlibdata, 
						 mb->background_skin))) {
    fprintf(stderr, "%s(%d): couldn't find image for background\n",
	    __FILE__, __LINE__);
    
    XUNLOCK(mb->display);
    return NULL;
  }

  private_data->mc              = (mrl_contents_t *) gui_xmalloc(sizeof(mrl_contents_t));
  private_data->mrls_num        = 0;
  private_data->last_mrl_source = NULL;

  hint.x      = mb->x;
  hint.y      = mb->y;
  hint.width  = private_data->bg_image->rgb_width;
  hint.height = private_data->bg_image->rgb_height;
  hint.flags  = PPosition | PSize;

  screen = DefaultScreen(mb->display);

  XAllocNamedColor(mb->display, 
                   DefaultColormap(mb->display, screen), 
		   "black", &black, &dummy);

  attr.override_redirect = True;
  attr.background_pixel  = black.pixel;
  /*
   * XXX:multivis
   * To avoid BadMatch errors on XCreateWindow:
   * If the parent and the new window have different depths, we must supply either
   * a BorderPixmap or a BorderPixel.
   * If the parent and the new window use different visuals, we must supply a
   * Colormap
   */
  attr.border_pixel      = 1;
  if (mb->imlibdata->x.visual != DefaultVisual(mb->display, screen)) {
    attr.colormap        = XCreateColormap(mb->display,
					   RootWindow(mb->display, screen),
					   mb->imlibdata->x.visual, AllocNone);
  } else {
    attr.colormap        = DefaultColormap (mb->display, screen);
  }

  private_data->window   = XCreateWindow (mb->display,
					  DefaultRootWindow(mb->display), 
					  hint.x, hint.y, hint.width, 
					  hint.height, 0, 
					  mb->imlibdata->x.depth, 
					  CopyFromParent, 
					  mb->imlibdata->x.visual,
					  CWBackPixel | CWBorderPixel | CWColormap, &attr);
  
  XSetStandardProperties(mb->display, private_data->window, title, title,
			 None, NULL, 0, &hint);

  XSelectInput(mb->display, private_data->window,
	       ButtonPressMask | ButtonReleaseMask | PointerMotionMask 
	       | KeyPressMask | ExposureMask | StructureNotifyMask);
  
  /*
   * wm, no border please
   */
  prop                 = XInternAtom(mb->display, "_MOTIF_WM_HINTS", True);
  mwmhints.flags       = MWM_HINTS_DECORATIONS;
  mwmhints.decorations = 0;
  
  XChangeProperty(mb->display, private_data->window, prop, prop, 32,
                  PropModeReplace, (unsigned char *) &mwmhints,
                  PROP_MWM_HINTS_ELEMENTS);						
  if(mb->window_trans != None)
    XSetTransientForHint (mb->display, private_data->window, mb->window_trans);

  /* set xclass */
  
  if((xclasshint = XAllocClassHint()) != NULL) {
    xclasshint->res_name  = mb->resource_name ? mb->resource_name : "xitk mrl browser";
    xclasshint->res_class = mb->resource_class ? mb->resource_class : "xitk";
    XSetClassHint(mb->display, private_data->window, xclasshint);
  }
  
  wm_hint = XAllocWMHints();
  if (wm_hint != NULL) {
    wm_hint->input         = True;
    wm_hint->initial_state = NormalState;
    wm_hint->flags         = InputHint | StateHint;
    XSetWMHints(mb->display, private_data->window, wm_hint);
    XFree(wm_hint);
  }
  
  gc = XCreateGC(mb->display, private_data->window, 0, 0);
  
  Imlib_apply_image(mb->imlibdata, 
		    private_data->bg_image, private_data->window);

  private_data->widget_list                = widget_list_new() ;
  private_data->widget_list->l             = gui_list_new ();
  private_data->widget_list->focusedWidget = NULL;
  private_data->widget_list->pressedWidget = NULL;
  private_data->widget_list->win           = private_data->window;
  private_data->widget_list->gc            = gc;
  
  lb.display        = mb->display;
  lb.imlibdata      = mb->imlibdata;
  lb.x              = mb->select.x;
  lb.y              = mb->select.y;
  lb.button_type    = CLICK_BUTTON;
  lb.label          = mb->select.caption;
  lb.callback       = mrlbrowser_select;
  lb.state_callback = NULL;
  lb.userdata       = (void *)private_data;
  lb.skin           = mb->select.skin_filename;
  lb.normcolor      = mb->select.normal_color;
  lb.focuscolor     = mb->select.focused_color;
  lb.clickcolor     = mb->select.clicked_color;
  lb.fontname       = mb->select.fontname;

  gui_list_append_content(private_data->widget_list->l,
			  label_button_create (&lb));

  lb.display        = mb->display;
  lb.imlibdata      = mb->imlibdata;
  lb.x              = mb->dismiss.x;
  lb.y              = mb->dismiss.y;
  lb.button_type    = CLICK_BUTTON;
  lb.label          = mb->dismiss.caption;
  lb.callback       = mrlbrowser_exit;
  lb.state_callback = NULL;
  lb.userdata       = (void *)mywidget;
  lb.skin           = mb->dismiss.skin_filename;
  lb.normcolor      = mb->dismiss.normal_color;
  lb.focuscolor     = mb->dismiss.focused_color;
  lb.clickcolor     = mb->dismiss.clicked_color;
  lb.fontname       = mb->dismiss.fontname;

  gui_list_append_content(private_data->widget_list->l,
			  label_button_create (&lb));
  
  private_data->add_callback      = mb->select.callback;
  private_data->kill_callback     = mb->kill.callback;
  mb->browser.dbl_click_callback  = handle_dbl_click;
  mb->browser.dbl_click_time      = DEFAULT_DBL_CLICK_TIME;
  mb->browser.userdata            = (void *)private_data;
  mb->browser.parent_wlist        = private_data->widget_list;
  gui_list_append_content (private_data->widget_list->l,
			   (private_data->mrlb_list = 
			    browser_create(&mb->browser)));

  lbl.display   = mb->display;
  lbl.imlibdata = mb->imlibdata;
  lbl.x         = mb->origin.x;
  lbl.y         = mb->origin.y;
  lbl.length    = mb->origin.max_length;
  lbl.label     = "";
  lbl.font      = mb->origin.skin_filename;
  lbl.animation = mb->origin.animation;
  lbl.window    = private_data->widget_list->win;
  lbl.gc        = private_data->widget_list->gc;
  
  gui_list_append_content(private_data->widget_list->l,
			  (private_data->widget_origin = 
			   label_create (&lbl)));
  
  memset(&private_data->current_origin, 0, strlen(private_data->current_origin));
  if(mb->origin.cur_origin)
    sprintf(private_data->current_origin, "%s", mb->origin.cur_origin);

  if(mb->ip_name.label.label_str) {

    lbl.display   = mb->display;
    lbl.imlibdata = mb->imlibdata;
    lbl.x         = mb->ip_name.label.x;
    lbl.y         = mb->ip_name.label.y;;
    lbl.font      = mb->ip_name.label.skin_filename;
    lbl.label     = mb->ip_name.label.label_str;
    lbl.length    = mb->ip_name.label.length;
    lbl.animation = mb->ip_name.label.animation;
    lbl.window    = private_data->widget_list->win;
    lbl.gc        = private_data->widget_list->gc;

    gui_list_append_content(private_data->widget_list->l, 
			    label_create (&lbl));
  }

  /*
   * Create buttons with input plugins names.
   */
  {
    int x, y;
    int i = 0;
    widget_t *tmp;
    
    x = mb->ip_name.button.x;
    y = mb->ip_name.button.y;
    
    while(mb->ip_availables[i] != NULL) {

      lb.display        = mb->display;
      lb.imlibdata      = mb->imlibdata;
      lb.x              = x;
      lb.y              = y;
      lb.button_type    = CLICK_BUTTON;
      lb.label          = mb->ip_availables[i];
      lb.callback       = mrlbrowser_grab_mrls;
      lb.state_callback = NULL;
      lb.userdata       = (void *)private_data;
      lb.skin           = mb->ip_name.button.skin_filename;
      lb.normcolor      = mb->ip_name.button.normal_color;
      lb.focuscolor     = mb->ip_name.button.focused_color;
      lb.clickcolor     = mb->ip_name.button.clicked_color;
      lb.fontname       = mb->ip_name.button.fontname;

      gui_list_append_content(private_data->widget_list->l,
			      (tmp = label_button_create (&lb)));

      y += widget_get_height(tmp) + 1;
      i++;
    }
  }
    
  private_data->visible     = 1;

  mywidget->enable          = 1;
  mywidget->running         = 1;
  mywidget->visible         = 1;
  mywidget->have_focus      = FOCUS_LOST;
  mywidget->x               = mb->x;
  mywidget->y               = mb->y;
  mywidget->width           = private_data->bg_image->width;
  mywidget->height          = private_data->bg_image->height;
  mywidget->widget_type     = WIDGET_TYPE_MRLBROWSER | WIDGET_TYPE_GROUP;
  mywidget->paint           = NULL;
  mywidget->notify_click    = NULL;
  mywidget->notify_focus    = NULL;
  mywidget->notify_keyevent = NULL;

  browser_update_list(private_data->mrlb_list, 
		      private_data->mc->mrls_disp, 
		      private_data->mrls_num, 0);
  
  XMapRaised(mb->display, private_data->window); 

  private_data->widget_key = 
    widget_register_event_handler("mrl browser",
				  private_data->window, 
				  mrlbrowser_handle_event,
				  NULL,
				  mb->dndcallback,
				  private_data->widget_list,
				  (void *) private_data);

  XUNLOCK (mb->display);

  return mywidget;
}

#endif
